@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <math.h>
 #include <float.h>
+#include <string.h>
 
 #include "nlopt-internal.h"
 
@@ -38,7 +39,9 @@
 
 #include "cdirect.h"
 
+#ifdef NLOPT_LUKSAN
 #include "luksan.h"
+#endif
 
 #include "crs.h"
 
@@ -393,8 +396,70 @@ static int elimdim_wrapcheck(nlopt_opt opt)
 }
 
 /*********************************************************************/
+/* wrapper functions for algorithms not considering all evaluated points as candidates
+ * or returning only the last evaluation which might not be optimum */
 
-#define POP(defaultpop) (opt->stochastic_population > 0 ? opt->stochastic_population : (nlopt_stochastic_population > 0 ? nlopt_stochastic_population : (defaultpop)))
+typedef struct {
+    nlopt_func f;
+    void *f_data;
+    const double *lb, *ub;      /* bounds, of length n */
+    double minf;
+    double *bestx;
+} memoize_data;
+
+
+static double memoize_func(unsigned n, const double *x, double *grad, void *d_)
+{
+    memoize_data *d = (memoize_data *) d_;
+    const double *lb = d->lb, *ub = d->ub;
+    double val;
+    unsigned i, feasible = 1;
+
+    val = d->f(n, x, grad, d->f_data);
+
+    for (i = 0; i < n; ++ i)
+    {
+        if (lb && (x[i] < lb[i]))
+            feasible = 0;
+        if (ub && (x[i] > ub[i]))
+            feasible = 0;
+    }
+    if (feasible && (val < d->minf))
+    {
+        d->minf = val;
+        memcpy(d->bestx, x, n * sizeof(double));
+    }
+    return val;
+}
+
+
+/* return whether to use memoize wrapping. */
+static int memoize_wrapcheck(nlopt_opt opt)
+{
+    if (!opt)
+        return 0;
+
+    /* constraints not supported */
+    if ((opt->m > 0) || (opt->p > 0))
+        return 0;
+
+    switch (opt->algorithm) {
+    case NLOPT_LN_COBYLA:
+    case NLOPT_LD_TNEWTON:
+    case NLOPT_LD_TNEWTON_RESTART:
+    case NLOPT_LD_TNEWTON_PRECOND:
+    case NLOPT_LD_TNEWTON_PRECOND_RESTART:
+        return 1;
+
+    default:
+        return 0;
+    }
+}
+
+
+/*********************************************************************/
+
+#define POP(defaultpop) (opt->stochastic_population > 0 ? (int)opt->stochastic_population : (nlopt_stochastic_population > 0 ? nlopt_stochastic_population : (defaultpop)))
 
 /* unlike nlopt_optimize() below, only handles minimization case */
 static nlopt_result nlopt_optimize_(nlopt_opt opt, double *x, double *minf)
@@ -532,7 +597,7 @@ static nlopt_result nlopt_optimize_(nlopt_opt opt, double *x, double *minf)
 #ifdef NLOPT_CXX
         if (!finite_domain(n, lb, ub))
             RETURN_ERR(NLOPT_INVALID_ARGS, opt, "finite domain required for global algorithm");
-        if (!stogo_minimize(ni, f, f_data, x, minf, lb, ub, &stop, algorithm == NLOPT_GD_STOGO ? 0 : (int) POP(2 * n)))
+        if (!stogo_minimize(ni, f, f_data, x, minf, lb, ub, &stop, algorithm == NLOPT_GD_STOGO ? 0 : POP(2 * (int)n)))
             return NLOPT_FAILURE;
         break;
 #else
@@ -590,22 +655,37 @@ static nlopt_result nlopt_optimize_(nlopt_opt opt, double *x, double *minf)
         }
 
     case NLOPT_LD_LBFGS:
+#ifdef NLOPT_LUKSAN
         return luksan_plis(ni, f, f_data, lb, ub, x, minf, &stop, opt->vector_storage);
+#else
+        printf("ERROR - attempting to use NLOPT_LD_LBFGS, but Luksan code disabled\n");
+        return NLOPT_INVALID_ARGS;
+#endif
 
     case NLOPT_LD_VAR1:
     case NLOPT_LD_VAR2:
+#ifdef NLOPT_LUKSAN
         return luksan_plip(ni, f, f_data, lb, ub, x, minf, &stop, opt->vector_storage, algorithm == NLOPT_LD_VAR1 ? 1 : 2);
+#else
+        printf("ERROR - attempting to use NLOPT_LD_VAR*, but Luksan code disabled\n");
+        return NLOPT_INVALID_ARGS;
+#endif
 
     case NLOPT_LD_TNEWTON:
     case NLOPT_LD_TNEWTON_RESTART:
     case NLOPT_LD_TNEWTON_PRECOND:
     case NLOPT_LD_TNEWTON_PRECOND_RESTART:
+#ifdef NLOPT_LUKSAN
         return luksan_pnet(ni, f, f_data, lb, ub, x, minf, &stop, opt->vector_storage, 1 + (algorithm - NLOPT_LD_TNEWTON) % 2, 1 + (algorithm - NLOPT_LD_TNEWTON) / 2);
+#else
+        printf("ERROR - attempting to use NLOPT_LD_TNEWTON*, but Luksan code disabled\n");
+        return NLOPT_INVALID_ARGS;
+#endif
 
     case NLOPT_GN_CRS2_LM:
         if (!finite_domain(n, lb, ub))
             RETURN_ERR(NLOPT_INVALID_ARGS, opt, "finite domain required for global algorithm");
-        return crs_minimize(ni, f, f_data, lb, ub, x, minf, &stop, (int) POP(0), 0);
+        return crs_minimize(ni, f, f_data, lb, ub, x, minf, &stop, POP(0), 0);
 
     case NLOPT_G_MLSL:
     case NLOPT_G_MLSL_LDS:
@@ -646,7 +726,7 @@ static nlopt_result nlopt_optimize_(nlopt_opt opt, double *x, double *minf)
                 nlopt_set_xtol_rel(local_opt, 1e-7);
             }
             push_force_stop_child(opt, local_opt);
-            ret = mlsl_minimize(ni, f, f_data, lb, ub, x, minf, &stop, local_opt, (int) POP(0), algorithm >= NLOPT_GN_MLSL_LDS && algorithm != NLOPT_G_MLSL);
+            ret = mlsl_minimize(ni, f, f_data, lb, ub, x, minf, &stop, local_opt, POP(0), algorithm >= NLOPT_GN_MLSL_LDS && algorithm != NLOPT_G_MLSL);
             pop_force_stop_child(opt);
             if (!opt->local_opt)
                 nlopt_destroy(local_opt);
@@ -675,7 +755,7 @@ static nlopt_result nlopt_optimize_(nlopt_opt opt, double *x, double *minf)
             nlopt_set_ftol_abs(dual_opt, nlopt_get_param(opt, "dual_ftol_abs", LO(ftol_abs, 0.0)));
             nlopt_set_xtol_rel(dual_opt, nlopt_get_param(opt, "dual_xtol_rel", 0.0));
             nlopt_set_xtol_abs1(dual_opt, nlopt_get_param(opt, "dual_xtol_abs", 0.0));
-            nlopt_set_maxeval(dual_opt, nlopt_get_param(opt, "dual_maxeval", LO(maxeval, 100000)));
+            nlopt_set_maxeval(dual_opt, (int)nlopt_get_param(opt, "dual_maxeval", LO(maxeval, 100000)));
 #undef LO
             if (algorithm == NLOPT_LD_MMA)
                 ret = mma_minimize(n, f, f_data, opt->m, opt->fc, lb, ub, x, minf, &stop, dual_opt, inner_maxeval, (unsigned)verbosity, rho_init, opt->dx);
@@ -793,7 +873,7 @@ static nlopt_result nlopt_optimize_(nlopt_opt opt, double *x, double *minf)
     case NLOPT_GN_ISRES:
         if (!finite_domain(n, lb, ub))
             RETURN_ERR(NLOPT_INVALID_ARGS, opt, "finite domain required for global algorithm");
-        return isres_minimize(ni, f, f_data, (int) (opt->m), opt->fc, (int) (opt->p), opt->h, lb, ub, x, minf, &stop, (int) POP(0));
+        return isres_minimize(ni, f, f_data, (int) (opt->m), opt->fc, (int) (opt->p), opt->h, lb, ub, x, minf, &stop, POP(0));
 
     case NLOPT_GN_ESCH:
         if (!finite_domain(n, lb, ub))
@@ -846,6 +926,7 @@ nlopt_result NLOPT_STDCALL nlopt_optimize(nlopt_opt opt, double *x, double *opt_
     void *f_data;
     nlopt_precond pre;
     f_max_data fmd;
+    memoize_data mmzd;
     int maximize;
     nlopt_result ret;
 
@@ -874,6 +955,18 @@ nlopt_result NLOPT_STDCALL nlopt_optimize(nlopt_opt opt, double *x, double *opt_
         opt->maximize = 0;
     }
 
+    if (memoize_wrapcheck(opt))
+    {
+        mmzd.f = opt->f;
+        mmzd.f_data = opt->f_data;
+        mmzd.lb = opt->lb;
+        mmzd.ub = opt->ub;
+        mmzd.minf = DBL_MAX;
+        mmzd.bestx = (double *) malloc(opt->n * sizeof(double));
+        opt->f = memoize_func;
+        opt->f_data = &mmzd;
+    }
+
     { /* possibly eliminate lb == ub dimensions for some algorithms */
         nlopt_opt elim_opt = opt;
         if (elimdim_wrapcheck(opt)) {
@@ -899,6 +992,16 @@ nlopt_result NLOPT_STDCALL nlopt_optimize(nlopt_opt opt, double *x, double *opt_
     }
 
   done:
+
+    if (memoize_wrapcheck(opt))
+    {
+        memcpy(x, mmzd.bestx, opt->n * sizeof(double));
+        free(mmzd.bestx);
+        *opt_f = mmzd.minf;
+        opt->f = mmzd.f;
+        opt->f_data = mmzd.f_data;
+    }
+
     if (maximize) { /* restore original signs */
         opt->maximize = maximize;
         opt->stopval = -opt->stopval;
